@@ -123,6 +123,8 @@ void start_playing() {
 	} else {
 		// FIXME display error
 	}
+
+	g_free(path);
 }
 
 void pause_playing() {
@@ -146,17 +148,22 @@ void update_status() {
 		g_string_truncate(s, 0);
 	}
 
-	const gchar *songname = "No file";
 	if (state != STATE_STOPPED) {
 		GtkTreeIter current;
 		make_iter(current_row, &current);
 
+		gchar *songname;
 		gtk_tree_model_get(GTK_TREE_MODEL(model), &current,
 		                   COLUMN_NAME, &songname, -1);
+
+		gchar *escaped = escape_for_markup(songname);
+		g_string_append_printf(s, "<b>%s</b>\n", escaped);
+
+		g_free(songname);
+		g_free(escaped);
+	} else {
+		g_string_append(s, "<b>No file</b>\n");
 	}
-	gchar *escaped = escape_for_markup(songname);
-	g_string_append_printf(s, "<b>%s</b>\n", escaped);
-	g_free(escaped);
 
 	switch (state) {
 	case STATE_STOPPED:
@@ -176,11 +183,10 @@ void update_status() {
 			g_string_append_printf(s, "%.0fkbps ", bitrate / 10);
 		}
 
-		g_string_append_printf(s, "%dch %dHz %dbit",
-		                       pstatus.in_fmt.channels, pstatus.in_fmt.rate, pstatus.in_fmt.bits);
+		append_format_str(s, &pstatus.in_fmt);
 		if (pstatus.converting) {
-			g_string_append_printf(s, " &#x2192; %dch %dHz %dbit",
-					       pstatus.out_fmt.channels, pstatus.out_fmt.rate, pstatus.out_fmt.bits);
+			g_string_append_printf(s, " &#x2192; ");
+			append_format_str(s, &pstatus.out_fmt);
 		}
 		break;
 	}
@@ -481,7 +487,9 @@ void add_recursively(const gchar *path) {
 	g_ptr_array_sort(files, compare);
 
 	for (int i = 0; i < files->len; i++) {
-		add_file((const gchar *) g_ptr_array_index(files, i));
+		gchar *file = g_ptr_array_index(files, i);
+		add_file(file);
+		g_free(file);
 	}
 
 	g_ptr_array_free(files, TRUE);
